/*******************************************************************************
  *
  *  program     : AbstractAccessPoint.java
  *  author      : Helge Janicke
  *  version     : 2001-01-20(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description : AbstractAccessPoint definiert grundlegende Methoden die zur
  *                Handhabung von AccessPoints bentigt werden.
  *                Ein AccessPoint ist ein Shape, das es dem Benutzer ermglicht
  *                Eigenschaften eines Objektes zu ndern. Zum Beispiel die
  *                Endpunkte einer Linie.
  *
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  s******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.Shape;
import java.awt.Stroke;
import java.awt.Paint;
import java.awt.BasicStroke;
import java.awt.Color;

/** Ein AccessPoint
 * ist ein Shape, das es dem Benutzer ermglicht Eigenschaften
 * eines Objektes zu ndern. Zum Beispiel die Endpunkte einer
 * Linie.<br>
 * Diese abstrakte Klasse definiert alle grundlegenden Funktionen die bentigt
 * werden, um einen AccessPoint zu benutzen. Es werden verschiedene Konstanten
 * zur Verfgung gestellt, die Auskunft ber den Verwendungszweck geben knnen.
 * Die hier definierten Methoden erlauben es den Typ eines AccessPoints zu
 * erfahren, und einen solchen grafisch darzustellen.
 */
public abstract class AbstractAccessPoint {

  /** Der Standart Linien Stil. */
  final static Stroke DEFAULT_STROKE = new BasicStroke();

  /** Die Standart Fllung. */
  final static Paint DEFAULT_PAINT = Color.green;

  /** Konstante die gesetzt ist, wenn kein Typ spezifiziert ist. */
  final static int NOT_INITIALIZED = -1;

  /** Konstante fr einen AccessPoint,
   * der sich zum aufziehen nach Nord eignet.
   */
  final static int SIZE_N = 0;

  /** Konstante fr einen AccessPoint,
   * der sich zum aufziehen nach Nord-Ost eignet.
   */
  final static int SIZE_NE = 1;

  /** Konstante fr einen AccessPoint,
   * der sich zum aufziehen nach Osten eignet.
   */
  final static int SIZE_E = 2;

  /** Konstante fr einen AccessPoint,
   * der sich zum aufziehen nach Sd-Ost eignet.
   */
  final static int SIZE_SE = 3;

  /** Konstante fr einen AccessPoint,
   * der sich zum aufziehen nach Sd eignet.
   */
  final static int SIZE_S = 4;

  /** Konstante fr einen AccessPoint,
   * der sich zum aufziehen nach Sd-West eignet.
   */
  final static int SIZE_SW = 5;

  /** Konstante fr einen AccessPoint,
   * der sich zum aufziehen nach West eignet.
   */
  final static int SIZE_W = 6;

  /** Konstante fr einen AccessPoint,
   * der sich zum aufziehen nach Nord-West eignet.
   */
  final static int SIZE_NW = 7;

  /** Konstante fr einen AccessPoint,
   * der sich zum Rotieren eignet.
   */
  final static int ROTATE = 9;

  /** Konstante fr einen AccessPoint,
   * der sich als Ankerpunkt eignet.
   */
  final static int ANCHOR_POINT = 10;

  /** Konstante fr einen AccessPoint,
   * der sich zum Scheren eignet.
   */
  final static int SHEAR = 11;

  /** Setzt den Typ des AccessPoint.
   * @param type die Konstante, die den Typ beschreibt.
   */
  public abstract void setType(int type);

  /** Liefert den aktuellen Typ des AccessPoints zurck.
   * @return Den aktuellen Typ.
   */
  public abstract int getType();

  /** Setzt das Aussehen des AccessPoints.
   * Das gesetzte Shape wird dargestellt.
   * @param appearance Das Aussehen des AccessPoints.
   */
  public abstract void setAppearance(Shape appearance);

  /** Liefert das Aussehen des AccessPoints zurck.
   * @return Das aktuelle Aussehen des AccessPoints.
   */
  public abstract Shape getAppearance();

  /** Liefert das aktuelle Paint Objekt, welches zum Zeichnen verwendet wird.
   * @return Das aktuelle Paint.
   */
  public abstract Paint getPaint();

  /** Liefert den aktuellen Linienstil zurck.*/
  public abstract Stroke getStroke();

  /** Gibt an, ob das Aussehen gefllt werden soll. */
  public abstract boolean isFilled();

}