/*******************************************************************************
  *
  *  program     : AbstractGrid.java
  *  author      : Helge Janicke
  *  version     : 2001-01-20(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description : AbstractGrid definiert grundlegende Eigenschaften, die ein
  *                Gitter erfllen mu. Das Prinzip besteht aus einer Koordinaten
  *                umsetzung von Punkten auf (diskrete) Bereiche, die den Gitter
  *                linien entsprechen.
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/

package jdraw;

/** AbstractGrid definiert grundlegende Eigenschaften, die ein
  * Gitter erfllen mu. Das Prinzip besteht aus einer Koordinaten
  * umsetzung von Punkten auf (diskrete) Bereiche, die den Gitter
  * linien entsprechen.
  */
public abstract class AbstractGrid {

  /** Flag das aussagt, ob das Gitter aktiv ist. */
  private boolean enabled=false;

  /** Flag das aussagt, ob das Gitter sichtbar ist. */
  private boolean visible=false;

  /** Erzeugt ein neues Standart Gitter. */
  public AbstractGrid() {
  }

  /** Liefert die nchste X Koordinate zurck.
    * @param x Die X Koordinate, die umgesetzt werden soll.
    * @return Die nchste gltige X Koordinate.
    */
  public abstract int getClosestX( int x );

  /** Liefert die nchste Y Koordinate zurck.
    * @param y Die Y Koordinate, die umgesetzt werden soll.
    * @return Die nchste gltige Y Koordinate.
    */
  public abstract int getClosestY( int y );

  /** Liefert das nchste passende MouseEvent zurck.
    * @param e Das MouseEvent, das umgesetzt werden soll.
    * @return Das nchste gltige MouseEvent.
    */
  public abstract java.awt.event.MouseEvent getClosestEvent( java.awt.event.MouseEvent e );

  /** Liefert einen Array von Shapes zurck, die das Gitter reprsentieren.
    * @param d Die Dimension des Gitters in Punkten.
    * @return Einen Array von Shapes, die das Gitter darstellen.
    */
  public abstract java.awt.Shape[] getShapes( java.awt.Dimension d );

  /** Liefert ob das Gitter an oder aus geschaltet ist.
    * @return <tt>true</tt> wenn das Gitter an ist.
    */
  public boolean isEnabled() { return this.enabled; }

  /** Schaltet das Gitter an/ aus.
    * @param flag <tt>true</tt> wenn Gitter angeschaltet werden soll.
    */
  public void setEnabled(boolean flag) { this.enabled = flag; }

  /** Liefert ob das Gitter sichtbar ist.
    * @return <tt>true</tt> wenn sichtbar.
    */
  public boolean isVisible() { return this.visible; }

  /** Setzt das visible flag.
    * @param <tt>true</tt> bedeutet sichbar.
    */
  public void setVisible(boolean flag) { this.visible = flag; }

}