/*******************************************************************************
  *
  *  program     : Drawable.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description : Ein Klasse, die das Drawable Interface implementiert mu
  *                Methoden zur Verfgung stellen, die das Zeichnen eines
  *                AbstractGraphObject ermglichen. Es werden solange
  *                handleMouse...() Methoden aufgerufen, bis eine der Methoden
  *                true zurckliefert. Danach wird die Methode getInstance()
  *                aufgerufen, die eine Instanz des AbstractGraphObjects zurck-
  *                liefert. Um einem Drawable anzuzeigen, da die Erzeugung
  *                eines Objekts abgebrochen werden soll wird die Methode
  *                handleTermination() aufgerufen. Das Drawable kann nun
  *                entscheiden, ob die gesammelten Daten ausreichend zur
  *                Erzeugung eines AbstractGraphObjects sind, oder nicht.
  *                Sind Die Daten nicht ausreichend, sollte getInstance() null
  *                zurckliefern.
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/

package jdraw;

import java.awt.event.*;
import java.awt.Shape;
import java.awt.Cursor;
import java.net.URL;
import javax.swing.JEditorPane;

/** Dieses Interface gibt der Klasse <tt>jdraw.UserDrawAction</tt>
  * die Mglichkeit eine Instanz der Klasse <tt>GraphObject</tt> dem Baum
  * zuzufgen. Ist in <tt>UserDrawAction</tt> der Modus so gesetzt,
  * da ein GraphObject gezeichnet werden soll, so werden die
  * <tt>handleMouse...()</tt> Methoden der unter <tt>drawable</tt> eingetragenen
  * Klasse aufgerufen, bis:
  * <UL>
  *   <LI> eine handleMouse...() Methode <tt>true</tt> zurckgibt.
  *   <LI> handleTermination() aufgerufen wird.
  * </UL>
  * Dies ermglicht einer implementierenden Klasse Informationen ber
  * das Erzeugen eines GraphObjects zu erlangen.
  * Sind alle zum erzeugen eines Objektes ntigen Informationen vorhanden
  * liefert die handleMouse...() Methode <tt>true</tt> zurck. Daraufhin
  * holt sich UserDrawAction eine Instanz dieser Klasse mit getInstance().
  * Wird der Zeichenvorgang abgebrochen, so wird die Methode
  * <tt>handleTermination()</tt> aufgerufen.
  * Auch wenn der Vorgang durch handleTermination() abgebrochen wurde wird
  * noch versucht die Instanz in den Baum einzuhngen.
  */
public interface Drawable {

  /** Behandelt ein MousePressed Event.
    * @param e Das MouseEvent aus UserDrawAction.
    * @return <tt>true</tt> wenn das Objekt erzeugt werden kann.
    *         <tt>false</tt> wenn noch mehr Informationen ntig sind.
    */
  public boolean handleMousePressed(MouseEvent e);

  /** Behandelt ein MouseDragged Event.
    * @param e Das MouseEvent aus UserDrawAction.
    * @return <tt>true</tt> wenn das Objekt erzeugt werden kann.
    *         <tt>false</tt> wenn noch mehr Informationen ntig sind.
    */
  public boolean handleMouseDragged(MouseEvent e);

  /** Behandelt ein MouseMoved Event.
    * @param e Das MouseEvent aus UserDrawAction.
    * @return <tt>true</tt> wenn das Objekt erzeugt werden kann.
    *         <tt>false</tt> wenn noch mehr Informationen ntig sind.
    */
  public boolean handleMouseMoved(MouseEvent e);

  /** Behandelt ein MouseReleased Event.
    * @param e Das MouseEvent aus UserDrawAction.
    * @return <tt>true</tt> wenn das Objekt erzeugt werden kann.
    *         <tt>false</tt> wenn noch mehr Informationen ntig sind.
    */
  public boolean handleMouseReleased(MouseEvent e);

  /** Behandelt ein MouseClicked Event.
    * @param e Das MouseEvent aus UserDrawAction.
    * @return <tt>true</tt> wenn das Objekt erzeugt werden kann.
    *         <tt>false</tt> wenn noch mehr Informationen ntig sind.
    */
  public boolean handleMouseClicked(MouseEvent e);

  /** Wird aufgerufen, bevor das Erzeugen des Objektes abgebrochen wird.
    * Diese Funktion sollte alle bisher gesammelten Informationen nutzen
    * um sie zu vervollstndigen, so das getInstance() in der Lage ist
    * ein Objekt zu erzeugen, oder falls dies nicht mglich ist die bisher
    * gesammelten Informationen lschen.
    */
  public void handleTermination();

  /** Wird aufgerufen, nachdem eine handleMouse...() Methode <tt>true</tt>
    * zurckgegeben hat, oder handleTermination() aufgerufen wurde.
    * @return Eine Instanz von GraphObject, die das gezeichnete Objekt
    *         reprsentiert. Wenn nicht erzeugt dann <tt>null</tt>.
    */
  public AbstractGraphObject getInstance();

  /** Diese Methode erlaubt dem Drawable auf eine Glasspane zurckzugreifen, auf
   * der sie die einzelnen Events graphisch darstellen kann.
   */
  public void setGlassPane(GlassPane glasspane);

  /** Der Cursor, der beim Zeichnen dieses Drawables angezeigt wird.
   * @return Den Cursor.
   */
  public Cursor supportCursor();

  public URL getHTMLHelp();

  public void setHTMLHelpPane( JEditorPane ep );
}