/*******************************************************************************
  *
  *  program     : GlassPane.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description :
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/

package jdraw;

import jdraw.AbstractGraphObject;
import jdraw.AccessPoint;
import java.awt.Paint;
import java.awt.Stroke;
import java.awt.Shape;
import java.awt.Component;
import java.awt.Cursor;

/** Eine GlassPane ist fr die Darstellung der Auswahl, sowie neu entstehender
  * Objekte zustndig. Sie verwaltet auch gleichzeitig die Auswahl. */
public interface GlassPane {

  /** Fgt der Auswahl ein Selectable hinzu.
    * @param o Das Selectable, welches hinzugefgt werden soll.
    */
  public void addSelection(Selectable o);

  /** Fgt der Auswahl ein Paint Objekt hinzu. Dieses wird in der paint() Methode gesetzt.
    * @param p Das Paint, das hinzugefgt werden soll.
    */
  public void addSelection(Paint p);

  /** Fgt der Auswahl ein Stroke Objekt hinzu. Dieses wird in der paint() Methode gesetzt.
    * @param s Das Stroke, das hinzugefgt werden soll.
    */
  public void addSelection(Stroke s);

  /** Entfernt ein Selectable aus der Auswahl.
    * @param o Das Selectable das entfernt werden soll.
    */
  public void remSelection(Selectable o);

  /** Entfernt ein Paint aus der Auswahl.
    * @param p Das Paint das entfernt werden soll.
    */
  public void remSelection(Paint p);

  /** Entfernt ein Stroke aus der Auswahl.
    * @param s Das Stroke das entfernt werden soll.
    */
  public void remSelection(Stroke s);

  /** Lscht die gesamte Auswahl. */
  public void clearSelection();

  /** Prft ob sich ein Selectable in der Auswahl befindet.
    * @param o Das zu prfende AbstractGraphObject.
    * @return <tt>true</tt> wenn es sich in der Auswahl befindet, sonst <tt>false</tt>.
    */
  public boolean isSelected(Selectable o);

  /** Fgt ein Shape hinzu, welches in der AbstractGlassPane gezeichnet wird.
    * Diese Shapes werden in der Reihenfolge in der sie eingetragen wurden gezeichnet und
    * erst nachdem die Auswahl gezeichnet wurde.
    * @param s Das Shape, das hinzugefgt wird.
    */
  public void addShape(Shape s);

  /** Entfernt ein Shape von der AbstractGlassPane.
    * Diese Shapes werden in der Reihenfolge in der sie eingetragen wurden gezeichnet und
    * erst nachdem die Auswahl gezeichnet wurde.
    * @param s Das Shape, das entfernt ewrden soll.
    */
  public void remShape(Shape s);

  /** Entfert alle zu zeichnenden Shapes. */
  public void clearShapes();

  /** Liefert die zur Auswahl hinzugefgten Objekte in der korrekten Reihenfolge.
    * @return Die zur Auswahl hinzugefgten Objekte.
    */
  public Object[] getSelection();

  /** Liefert das ausgewhlte AbstractGraphObject zurck.
    * Falls mehr als ein Objekt ausgewhlt ist, liefert es eine Instanz der Klasse
    * ImplicitGroup.
    * @return Das ausgewhlte AbstractGraphGroup Objekt.
    */
  public Selectable getSelectable();

  /** Zeichnet die GlassPane neu und die als refreshComponent angegebene Komponente.*/
  public void refresh();

  /** Setzt die Refresh Komponente.
    * @param c Das Component Objekt dessen paint() Methode aufgerufen werden soll, wenn ein refresh request kommt. */
  public void setRefreshComponent(Component c);

  /** Setzt den Cursor fr die Glasspane um. */
  public void setCursor(Cursor c);

}