/*******************************************************************************
  *
  *  program     : Layer.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description :
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.*;

/** Diese Klasse reprsentiert eine Ebene. */
public class Layer {

  private boolean active = false;
  private boolean visible = true;
  private AlphaComposite alpha = AlphaComposite.SrcOver;
  private String name = "";

  /** Erzeugt eine neue Ebene mit dem angegebenen Namen.
    * @param name der Name, der der Ebene zugewiesen wird. */
  public Layer(String name) {
    this.name = name;
    setAlpha(AlphaComposite.SRC_OVER, 0.5f);
  }

  /** Setzt den Alphawert der Ebene.
    * @param rule Die Vorschrift, mit der das alpha angewendet werden soll.
    * @param alpha der Alphawert (0.0;1.0]
    */
  public void setAlpha(int rule, float alpha) {
    this.alpha = AlphaComposite.Src;
    this.alpha = this.alpha.getInstance(rule, alpha);
  }

  /** Liefert das Alphpacomposite dieser Ebene. */
  public Composite getAlpha() { return (Composite) alpha; }

  /** Setzt diese Ebene auf aktiv.
    * Es kann nur auf aktive ebenen gezeichnet werden.
    * @param active <tt>true</tt> wenn Ebene activ sein soll.
    */
  public void setActive(boolean active) {
    this.active = active;
  }

  /** Liefert ob Ebene aktiv ist.
    * @return <tt>true</tt> wenn ebene aktiv.
    */
  public boolean isActive() { return this.active; }

  /** Setzt das visible flag dieser Ebene.
    * @param visible <tt>true</tt> wenn Ebene sichbar sein soll.
    */
  public void setVisible(boolean visible) {
    this.visible = visible;
  }

  /** Liefert das visible flag der Ebene.
    * @return <tt>true</tt> wenn Eben sichtbar.
    */
  public boolean isVisible() { return this.visible; }

  /** Liefert eine einfache Beschreibung der Ebene. */
  public String toString() { return name + ((visible) ? " visible" : " invisible") + " alpha=" + alpha.getAlpha(); }

}