/*******************************************************************************
  *
  *  program     : LayerPainter.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description :
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.geom.*;
import javax.swing.tree.*;
import java.util.*;
import java.awt.event.*;
import javax.swing.*;
import java.awt.*;

/** LayerPainter stellt die Zeichnung ab einem bestimmten Knoten im Baum dar.
  * Alle Elemente die ein AbstractGraphObjekt sind werden gezeichnet.
  * Dazu wird der Unterbaum in preorder rehenfolge durchlaufen. Dies
  * gewhrleistet, das die einzelnen Zeichenobjekte auch in der Reihenfolge
  * gezeichnet werden, in der sie in den Baum eingefgt wurden.
  * Vor dem Zeichnen einer Ebene wird deren Alphawert gesetzt. Vor dem Zeichnen
  * der einzelnen Zeichenobjekte werden deren Fllfarben, Linienstil und
  * Linienfarbe gesetzt.
  */
public class LayerPainter extends JLabel {

  // Mu noch an die Bildgre angepasst werden!!!
  private int height = 1000;
  private int width = 800;

  /** Der Knoten, ab dem gezeichnet wird. */
  private DefaultMutableTreeNode node;

  public Graphics2D g2 ;

  /** Standartkonstruktor. */
  public LayerPainter() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /** Konstruktor, der einen Anzeigebereich setzt.
   * @param node Der Baumknoten ab dem gezeichnet wird.
   */
  public LayerPainter(DefaultMutableTreeNode node) {
    try {
      setNode(node);
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /** Zeichnet den Baum ab der entsprechenden Stelle (node) */
  public void paintComponent (Graphics g) {

    // casten.
    g2 = (Graphics2D) g;
    boolean visible=true;

    g2.setColor(new Color(250,250,240));
    g2.fillRect(0,0,width, height);
    // Wenn berhaupt etwas zu zeichnen ist...
    if( node != null ) {

      // Den Baum in der PreOrder- Reihenfolge durchlaufen...
      for (Enumeration e = getNode().preorderEnumeration(); e.hasMoreElements(); ) {
        try {

          // Die Nutzdaten aus dem Baumknoten holen.
          Object obj = ((DefaultMutableTreeNode) e.nextElement()).getUserObject();

          // Wenn Nutzdaten...
          if (obj != null) {

            if (obj instanceof Layer) {
              g2.setComposite(((Layer) obj).getAlpha());
              visible = ((Layer) obj).isVisible();
            }

            // Wenn ein zeichnbares Objekt...
            if (visible) {
              if (obj instanceof AbstractGraphObject) {

                // System.err.println("try drawing AbstractGraphObject");

                // casten.
                AbstractGraphObject grph = (AbstractGraphObject) obj;

                // Den Strich fr dieses Objekt setzen.
                g2.setStroke(grph.getStroke());

                // affine Transformation ausfhren.
                AffineTransform old = g2.getTransform();

                if (grph.isFilled()) {
                  g2.setPaint(grph.getFillPaint());
                  g2.fill(grph);
                }

                if (grph.isOutlined()) {
                  g2.setPaint(grph.getLinePaint());
                  g2.draw(grph);
                }

                g2.setTransform(old);
              }
            }
          }
        } catch (ClassCastException exc) {
         // Fehler... kann eigentlich nicht vorkommen, aber besser ist das!
        }
      }
    }
  }

  private void jbInit() throws Exception {
    this.setBackground(Color.white);
    this.setOpaque(true);
    this.setLayout(null);
  }

  /** Weist dem LayerPainter einen Bereich des Bildes zu.
   * @param node Der Knoten, dessen KindKnoten gezeichnet werden sollen.
   */
  public void setNode (DefaultMutableTreeNode node) {
    this.node = node;
  }

  /** Liefert den Baumknoten, der angezeigt wird.
   * @return Den Knoten, ab dem gezeichnet wird.
   */
  public DefaultMutableTreeNode getNode() {
    return node;
  }

}