/*******************************************************************************
  *
  *  program     : SimpleAccessPoint.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description :
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.Stroke;
import java.awt.Shape;
import java.awt.Paint;
import java.awt.geom.Rectangle2D;

/** Dies ist eine einfache Implementierung eines AbstractAccessPoint.
  */
public class SimpleAccessPoint extends AbstractAccessPoint {
  private int type = AbstractAccessPoint.NOT_INITIALIZED;
  private String command = "";
  private Shape appearance = new Rectangle2D.Double(0,0,4,4);

  /** Standartkonstruktor. */
  public SimpleAccessPoint() {
  }

  /** Erzeugt einen SimpleAccessPoint des angegebenen Typs.
    * @param type Der Typ (Verwendungszweck) des Accesspoints.
    */
  public SimpleAccessPoint(int type) {
    setType(type);
  }

  /** Erzeugt einen SimpleAccessPoint mit dem angegebenen Typ und Commando.
    * Der command string dient zur einfachen Unterscheidung von verschiedenen
    * AccessPoints gleichen Typs.
    * @param type Der Typ (Verwendungszweck) des Accesspoints.
    * @param cmd Der command string.
    */
  public SimpleAccessPoint(int type, String cmd) {
    this(type);
    setCommand(cmd);
  }

  public void setType(int type) {
    this.type = type;
  }

  public int getType() {
    return this.type;
  }

  public Stroke getStroke() {
    return AbstractAccessPoint.DEFAULT_STROKE;
  }

  public Shape getAppearance() {
    return this.appearance;
  }

  public Paint getPaint() {
    return AbstractAccessPoint.DEFAULT_PAINT;
  }

  public void setAppearance(Shape s) {
    this.appearance = s;
  }

  public boolean isFilled() {
    return true;
  }

  public void setCommand(String cmd) {
    this.command = cmd;
  }

  public String getCommand() {
    return this.command;
  }

  public String toString() {
    return "SimpleAccessPoint(" + type + "; " + command + ")";
  }

}