/*******************************************************************************
  *
  *  program     : SimpleGraphObject.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description :
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.Rectangle;
import java.awt.Paint;
import java.awt.geom.Point2D;
import java.awt.geom.PathIterator;
import java.awt.geom.AffineTransform;
import java.awt.Stroke;
import java.awt.geom.Rectangle2D;
import java.awt.Shape;
import java.awt.Color;
import java.awt.BasicStroke;

/** Dies ist eine einfache Implementierung des AbstractGraphObject.
  * Hier werden Methoden fr das setzen einer Affinen Transformation und
  * von Farbe, Fllung und Linienstil vordefiniert.
  * Beim Schreiben eigener Zeichengerte empfiehlt es sich diese von der Klasse
  * SimpleGraphObject abzuleiten.
  */
public class SimpleGraphObject extends AbstractGraphObject {

  protected Shape shape=null;
  private AffineTransform affineTransform = new AffineTransform();
  private Paint linePaint = null;
  private Paint fillPaint = null;
  private Stroke stroke = null;
  private boolean filled=false;
  private boolean outlined=true;

  public SimpleGraphObject() {
  }

  protected void setShape(Shape s) {
    this.shape = s;
  }

  protected Shape getShape() {
    return this.shape;
  }

  public void setAffineTransform(AffineTransform at) {
    this.affineTransform = at;
  }

  public AffineTransform getAffineTransform() {
    return this.affineTransform;
  }

  public void setFilled(boolean flag) {
    this.filled = flag;
  }

  public boolean isFilled() {
    return this.filled;
  }

  public void setOutlined(boolean flag) {
    this.outlined = true;
  }

  public boolean isOutlined() {
    return this.outlined;
  }

  public void setLinePaint(Paint p) {
    this.linePaint = p;
  }

  public Paint getLinePaint() {
    return (this.linePaint==null) ? AbstractGraphObject.DEFAULT_LINEPAINT : this.linePaint;
  }

  public void setFillPaint(Paint p) {
    this.fillPaint = p;
  }

  public Paint getFillPaint() {
    return (this.fillPaint==null) ? AbstractGraphObject.DEFAULT_FILLPAINT : this.fillPaint;
  }

  public void setStroke(Stroke s) {
    this.stroke = s;
  }

  public Stroke getStroke() {
    return (this.stroke==null) ? AbstractGraphObject.DEFAULT_STROKE : this.stroke;
  }

  public Rectangle getBounds() {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.getBounds();
  }

  public Rectangle2D getBounds2D() {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.getBounds2D();
  }

  public boolean contains(double x, double y) {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.contains(x,y);
  }

  public boolean contains(Point2D p) {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.contains(p);
  }

  public boolean contains(double x, double y, double w, double h) {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.contains(x,y,w,h);
  }

  public boolean contains(Rectangle2D r) {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.contains(r);
  }

  public PathIterator getPathIterator(AffineTransform at, double flatness) {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.getPathIterator(at,flatness);
  }

  public PathIterator getPathIterator(AffineTransform at) {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.getPathIterator(at);
  }

  public boolean intersects(double x, double y, double w, double h) {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.intersects(x,y,w,h);
  }

  public boolean intersects(Rectangle2D r) {
    Shape s = getAffineTransform().createTransformedShape(this.shape);
    return s.intersects(r);
  }

}