/*******************************************************************************
  *
  *  program     : SimpleGrid.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description : Ein einfaches Zeichen-Programm in Java, da mit mehreren
  *                Ebenen arbeitet. Die einzelnen Ebenen knnen mit alpha-
  *                blending kombiniert werden.<p>
  *
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.event.MouseEvent;
import java.awt.Shape;
import java.awt.Dimension;

import java.awt.*;
import javax.swing.*;

/** This class specifies a grid that can be used to draw on gridpoints only.
 *  All classes that will provide such a functionality should subclass SimpleGrid.
 */
public class SimpleGrid extends AbstractGrid {

  /** the intervalls per unit. */
  private int intervalls = 1;

  /** The unit used by this grid. */
  private Unit unit = Unit.INCH_UNIT;

  /** The delta in dots from one gridpoint to the next one. */
  private double delta;

  /** Constructs a new SimpleGrid. */
  public SimpleGrid() {
    setUnit(Unit.INCH_UNIT);
  }

  /** Returns the closest matching MouseEvent.
   *  MouseEvent.getX() and MouseEvent.getY() are interpreted as dots.
   *  @param e The MouseEvent given.
   *  @return The MouseEvent with the closest X and Y coordinate.
   */
  public MouseEvent getClosestEvent(MouseEvent e) {
    if (!isEnabled()) return e;
    MouseEvent evt = new MouseEvent((Component) e.getSource(), e.getID(), e.getWhen(), e.getModifiers(),
      getClosestX(e.getX()), getClosestY(e.getY()), e.getClickCount(), e.isPopupTrigger());
    return evt;
  }

  /** Returns an array of shapes in the specified range.
   *  @param d The Dimesion for the grid.
   *  @return The array of shapes to display the grid.
   */
  public Shape[] getShapes(Dimension d) {
    return null;
  }

  /** Returns the best matching y' grid position for a given y.
   *  @param y in dots.
   *  @return The best matching y' in dots.
   */
  public int getClosestY(int y) {
    if (!isEnabled()) return y;
    return (int) ((Math.round((double) y / delta)) * unit.getOneUnitInDot());
  }

  /** Returns the best matching x' grid position for a given x.
   *  @param x in dots.
   *  @return The best matching x' in dots.
   */
  public int getClosestX(int x) {
    if (!isEnabled()) return x;
    return (int) ((Math.round((double) x / delta)) * unit.getOneUnitInDot());
  }

  /** Applies a Unit to the grid.
   *  @param the unit.
   */
  public void setUnit( Unit unit ) {
    this.unit = unit;
    delta = unit.getOneUnitInDot() / intervalls;
    System.err.println("delta = " + delta);
  }

  /** returns the current unit.
   *  @return the current unit.
   */
  public Unit getUnit() {
    return this.unit;
  }

}