/*******************************************************************************
  *
  *  program     : Transformable.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description :
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/

package jdraw;

import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.Shape;

/** Dieses Interface definiert die Methoden, die bentigt werden, um
  * ein GraphObject im nachhinein durch den Benutzer ndern zu knnen.
  * Es ermglicht der UserDrawAction auch ein anderes Shape zu zeichnen
  * wenn das Transformable selektiert, bzw. bewegt wird.
  */
public interface Transformable {

  /** Stellt die fr den mit <tt>usage</tt> angegebenen Modus die
    * AccessPoints bereit.
    * @param usage Siehe AccessPoint.USAGE_...
    * @return die AccessPoint's fr den Modus. null fr keine.
    */
  public AbstractAccessPoint[] getAccessPoints(int usage);

  /** Stellt die AccessPoints bereit.
    * @param usage Siehe AccessPoint.USAGE_...
    * @return die AccessPoint's fr den Modus. null fr keine.
    */
  public AbstractAccessPoint[] getAccessPoints();

  /** Wird aufgerufen, wenn die AccessPoints nicht mehr bentigt werden.
   * Dies Methode sollte die eigenen Referenzen auf die AccessPoints lschen!
   */
  public void freeAccessPoints();

  /** Wird aufgerufen, wenn ein AccessPoint angeklickt wurde.
    * @param e Das MouseEvent aus UserdrawAction.
    * @param ap Der AccessPoint.
    */
  public void handleAccessPointPressed(MouseEvent e, AbstractAccessPoint ap);

  /** Wird aufgerufen, wenn ein AccessPoint verschoben wurde.
    * Ist dafr zustndig das Transformable anzupassen, sowie den (die)
    * AccessPoint(s) zu verschieben.
    * @param e Das MouseEvent aus UserdrawAction.
    * @param ap Der AccessPoint.
    */
  public void handleAccessPointDragged(MouseEvent e, AbstractAccessPoint ap);

  /** Wird aufgerufen, wenn ein AccessPoint losgelassen wurde.
    * @param e Das MouseEvent aus UserdrawAction.
    * @param ap Der AccessPoint.
    */
  public void handleAccessPointReleased(MouseEvent e, AbstractAccessPoint ap);

  /** Wird aufgerufen, wenn sich das Transformable in einer Gruppe
    * (implizit oder explizit) befindet. Es ist dafr zustndig die
    * affine Transformation auf das Transformable anzuwenden.
    * @param at Die affine Transformation.
    * @param groupBounds Die Boundingbox der Gruppe.
    */
  public void handleGroupTransformation(AffineTransform at, Rectangle2D groupBounds);

  public void handleDrag(MouseEvent e, Point2D startPoint);
  public void handleDrop(MouseEvent e, Point2D startPoint);


  /** Gibt an, ob das Transformable in einer Gruppe Transformiert werden
    * kann.
    * @return <tt>true</tt> wenn in Gruppe transformierbar.
    */
  public boolean isGroupTransformationSupported();

  /** Liefert das Shape, das gezeichnet wird, wenn das Transformable bewegt  * wird.
    * @return Das Shape, das beim Bewegen des Transformables gezeigt wird.
    */
  public Shape supportDragShape();

  /** Liefert das Shape, das gezeichnet wird, wenn das Transformable
    * selektiert wird.
    * @return Das Shape, das bei der Auswahl des Transformables gezeigt
    *         wird.
    */
  public Shape supportSelectedShape();

}
