/*******************************************************************************
  *
  *  program     : Unit.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description : Ein einfaches Zeichen-Programm in Java, da mit mehreren
  *                Ebenen arbeitet. Die einzelnen Ebenen knnen mit alpha-
  *                blending kombiniert werden.<p>
  *
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.Toolkit;

/** This class holds the information of a unit.
 *  To construct a new unit you hav to specify a name and a factor that specifies
 *  how many units are one inch. This value can later be retrieved using getUnitInInch().
 */
public class Unit {

  /** The screen resolution in inch. See java.awt.Toolkit */
  private final static int INCH = Toolkit.getDefaultToolkit().getScreenResolution();

  /** The name of the unit. */
  protected String name;

  public final static Unit INCH_UNIT = new Unit("in",1.0);
  public final static Unit CM_UNIT   = new Unit("cm",2.54);
  public final static Unit MM_UNIT   = new Unit("mm",25.4);

  /** The factor to multiply the unit with to get one inch. */
  protected double oneUnitInInch;
  protected double oneInchInUnit;
  protected double oneUnitInDot;
  protected double oneDotInUnit;

  /** Constructs a new unit.
   *  @param name The name of the unit.
   *  @param inInch The factor to multiply the unit with to get one inch.
   */
  public Unit(String name, double oneInchInUnit) {
    this.name = name;
    this.oneInchInUnit = oneInchInUnit;
    this.oneUnitInInch = 1 / oneInchInUnit;
    this.oneUnitInDot = this.oneUnitInInch * INCH;
    this.oneDotInUnit = 1 / this.oneUnitInDot;
    System.err.println("\n name = " + name);
    System.err.println("one inch in unit = " + this.oneInchInUnit + "\n" +
    "one unit in inch = " + this.oneUnitInInch + "\n" +
    "one Unit in dot  = " + this.oneUnitInDot + "\n" +
    "one dot in unit  = " +  this.oneDotInUnit
    );
  }

  /** Returns the name of the unit.
   *  @return The name of the unit.
   */
  public String getName() {
    return this.name;
  }

  /** Returns the factor to multiply inch with to get one unit.
   *  @return The factor.
   */
  public double getOneUnitInInch() {
    return this.oneUnitInInch;
  }

  /** Returns the factor to multiply a unit with to get one inch.
   *  @return The factor.
   */
  public double getOneInchInUnit() {
    return this.oneInchInUnit;
  }

  /** Returns the factor to multiply a dot with to get one unit.
   *  @return The factor.
   */
  public double getOneUnitInDot() {
    return this.oneUnitInDot;
  }

  /** Returns the factor to multiply a unit with to get one dot.
   *  @return The factor.
   */
  public double getOneDotInUnit() {
    return this.oneDotInUnit;
  }

  /** Returns the value in inch.
   *  @param value The value in units.
   *  @return The value in inch.
   */
  public double unitToInch( double value ) {
    return  value * this.oneUnitInInch;
  }

  /** Returns the value in units.
   *  @param value The value in inch.
   *  @return The value in units.
   */
  public double inchToUnit( double value ) {
    return value * oneInchInUnit;
  }

  /** Returns the value in dots.
   *  @param value The value in units.
   *  @return The value in dots.
   */
  public double unitToDot( double value ) {
    return value * this.oneUnitInDot;
  }

  /** Returns the value in units.
   *  @param value The value in dots.
   *  @return The value in units.
   */
  public double dotToUnit( double value ) {
    return value * this.oneDotInUnit;
  }

}