/*******************************************************************************
  *
  *  program     : jdLine.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description :
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.geom.Line2D;
import java.awt.event.MouseEvent;
import java.awt.geom.AffineTransform;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Point2D;
import java.awt.Shape;
import javax.swing.*;

/** Diese Klasse reprsentiert eine Linie.
 *  Alle Transformationen werden durch Punktberechnungen durchgefhrt.
 *  Zum Erzeugen von Linien wird die Klasse jdLineGenerator verwendet,
 *  deren getInstance() Methode eine Instanz dieser Klasse zurckliefert.
 */
public class jdLine extends SimpleGraphObject implements Transformable, Infoable {
  /** Die Linie, wie sie verwendet wird, also das Shape-Object von SimpleGraphObject */
  private Line2D line;

  /** Die Linie, die als DraggedShape angezeigt wird.*/
  private Line2D dragLine;

  /** Die AccessPoints, ber die eine Linie transformiert werden kann. */
  private SimpleAccessPoint[] ap = null;

  /** Ein Punkt, der beim Durchfhren von Transformationen, zum Speichern des
   *  Anfangspunkt benutz wird.
   */
  private Point2D startPoint;

  /** Label, da im InfoPanel die Breite angibt, (P2.x - P1.x) */
  static JLabel lblWidth = new JLabel();

  /** Label, da den Winkel in Grad zur x-Achse angibt */
  static JLabel lblAngle = new JLabel();

  /** Label, da die Entfernung zwischen den beiden Punkten angibt */
  static JLabel lblDistance = new JLabel();

  /** Label, da die Hhe angibt (P2.y - P1.y) */
  static JLabel lblHeight = new JLabel();

  /** Das InfoPanel, das zum anzeigen von Linieneigenschaften verwendet wird. */
  static JPanel infoPanel = new JPanel();

  /** Label, da den ersten Punkt beschreibt */
  static JLabel lblP2 = new JLabel();

  /** Label, da den zweiten Punkt beschreibt */
  static JLabel lblP1 = new JLabel();

  // static initializer
  static {
    infoPanel.add(lblP1, null);
    infoPanel.add(lblP2, null);
    infoPanel.add(lblAngle, null);
    infoPanel.add(lblDistance, null);
    infoPanel.add(lblWidth, null);
    infoPanel.add(lblHeight, null);
    infoPanel.setBackground(java.awt.Color.lightGray);
    infoPanel.setOpaque(true);
  }

  /** Erzeugt ein neues jdLine Objekt.
   *  @param x1 Die x-Koordinate des ersten Punkts.
   *  @param y1 Die y-Koordinate des ersten Punkts.
   *  @param x2 Die x-Koordinate des zweiten Punkts.
   *  @param y2 Die x-Koordinate des zweiten Punkts.
   */
  public jdLine(double x1, double y1, double x2, double y2) {
    this.line = new Line2D.Double(x1,y1,x2,y2);
    setShape(line);
  }

  /** Erzeugt ein neues jdLine Objekt.
   *  @param line Die Linie die erzeugt werden soll.
   */
  public jdLine(Line2D line) {
    this.line = line;
    setShape(line);
  }

  /** Liefert eine einfache Beschreibung der Linie. */
  public String toString() {
    return "Line (x1=" + line.getX1() + "; y1=" + line.getY1() + "; x2=" +
            line.getX2() + "; y2=" + line.getY2() +")";
  }

  /** Stellt die AccessPoints fr die Linie zur Verfgung.
    * Dies sind zwei AccessPoints an den Endpunkten der Linie. Diese sind vom Typ
    * ANCHOR_POINT und werden als Rechteck dargestellt.
    */
  public AbstractAccessPoint[] getAccessPoints(int usage) {
    if (ap==null) {
      ap = new SimpleAccessPoint[2];

      switch (usage) {
        case UserDrawAction.SELECTMODE:
          ap[0] = new SimpleAccessPoint(SimpleAccessPoint.ANCHOR_POINT,"point1");
          ap[1] = new SimpleAccessPoint(SimpleAccessPoint.ANCHOR_POINT,"point2");
          break;
        default :
          freeAccessPoints();
      }
    }
    adjustAccessPoints();
    return ap;
  }

  /** Justiert die AccessPoints an die Linie.
    * Dies ist zum Beispiel nach dem Verschieben notwendig.
    */
  private void adjustAccessPoints() {
    Point2D p1 = line.getP1();
    Point2D p2 = line.getP2();

    getAffineTransform().transform(p1,p1);
    getAffineTransform().transform(p2,p2);

    ap[0].setAppearance(new Rectangle2D.Double(p1.getX(), p1.getY(), 4, 4));
    ap[1].setAppearance(new Rectangle2D.Double(p2.getX(), p2.getY(), 4, 4));
  }

  public AbstractAccessPoint[] getAccessPoints() {
    if (ap==null) getAccessPoints(UserDrawAction.SELECTMODE);
    adjustAccessPoints();
    return ap;
  }

  public void freeAccessPoints() {
    ap = null;
  }

  public void handleAccessPointPressed(MouseEvent e, AbstractAccessPoint ap) {
    System.err.println("SimpleAccessPoint pressed : " + ap );
    startPoint = e.getPoint();
    dragLine = (Line2D) line.clone();
  }

  public void handleAccessPointDragged(MouseEvent e, AbstractAccessPoint ap) {
    SimpleAccessPoint sap = (SimpleAccessPoint) ap;
    Rectangle2D r = (Rectangle2D) sap.getAppearance();
    double deltaX = e.getX() - startPoint.getX();
    double deltaY = e.getY() - startPoint.getY();

    // Denn Accesspunkt anpassen
    r.setRect(r.getX() + deltaX, r.getY() + deltaY, r.getWidth() , r.getHeight());

    // Die Linie anpassen
    if (sap.getCommand().equals("point1")) {
      dragLine.setLine(line.getX1() + deltaX, line.getY1() + deltaY , line.getX2(), line.getY2());
    } else if (sap.getCommand().equals("point2")) {
      dragLine.setLine(line.getX1(), line.getY1(), line.getX2()+ deltaX, line.getY2() + deltaY);
    }

  }

  public void handleAccessPointReleased(MouseEvent e, AbstractAccessPoint ap) {
    System.err.println("released AccessPoint: " + ap);
    line.setLine(dragLine);
  }

  public void handleGroupTransformation(AffineTransform at, Rectangle2D groupBounds) {
    if (at.getType() == AffineTransform.TYPE_TRANSLATION) {
      this.getAffineTransform().concatenate(at);
    }
  }

  public void handleDrag(MouseEvent e, Point2D startPoint) {
    // System.err.println("dragged!!!");
    double deltaX = e.getX() - startPoint.getX();
    double deltaY = e.getY() - startPoint.getY();
    if (dragLine == null) dragLine = new Line2D.Double();
    dragLine.setLine(line.getX1() + deltaX, line.getY1() + deltaY,
                 line.getX2() + deltaX, line.getY2() + deltaY);
  }

  public void handleDrop(MouseEvent e, Point2D startPoint) {
    line.setLine(dragLine);
    dragLine = null;
  }

  public boolean isGroupTransformationSupported() {
    return true;
    //TODO: Implement this jdraw.Transformable method
  }

  public Shape supportDragShape() {
    return getAffineTransform().createTransformedShape(dragLine);
    //TODO: Implement this jdraw.Transformable method
  }

  public Shape supportSelectedShape() {
    return null;
    //TODO: Implement this jdraw.Transformable method
  }

  public JPanel getInfoPanel() {
    // System.err.println("hello");
    return infoPanel;
  }

  public void updateInfoPanel() {
    updateInfoPanel(this);
  }

  public void updateInfoPanel(AbstractGraphObject o) {
    if (o instanceof jdLine) {
      if (this.dragLine != null ) {
        jdLine line = (jdLine) o;
        lblHeight.setText("Hhe=");
        lblDistance.setText("Abstand=");
        lblAngle.setText("Winkel=");
        lblWidth.setText("Breite=");
        lblP2.setText("P2(" + this.dragLine.getX2()+ "; " + this.dragLine.getY2()+ ")" );
        lblP1.setText("P1(" + this.dragLine.getX1()+ "; " + this.dragLine.getY1()+ ")" );
      } else {
        jdLine line = (jdLine) o;
        lblHeight.setText("Hhe=");
        lblDistance.setText("Abstand=");
        lblAngle.setText("Winkel=");
        lblWidth.setText("Breite=");
        lblP2.setText("P2(" + this.line.getX2()+ "; " + this.line.getY2()+ ")" );
        lblP1.setText("P1(" + this.line.getX1()+ "; " + this.line.getY1()+ ")" );
      }
    } else {
      throw new IllegalArgumentException("can't show info: not a jdLine Object");
    }
  }

}