/*******************************************************************************
  *
  *  program     : jdLineGenerator.java
  *  author      : Helge Janicke
  *  version     : 2000-10-19(001)
  *  (c)         : Helge Janicke, Karsten Wolke, Niels-Peter de Witt
  *  email       : heljanic@agent.fho-emden.de
  *  description : Dies ist die Klasse, die fr das Zeichnen der Linie zustndig
  *                ist. Hier werden die Methoden, die in Drawable definiert sind
  *                implementiert.
  *
  ******************************************************************************
  * modifications:
  * date: ******* who: ****** description: *************************************
  * ----------------------------------------------------------------------------
  * 2000-10-19    heljanic    your description here
  * ----------------------------------------------------------------------------
  ******************************************************************************
  * comments:
  *
  ******************************************************************************
  * license:
  *
*******************************************************************************/
package jdraw;

import java.awt.event.MouseEvent;
import java.awt.geom.Line2D;
import java.awt.Cursor;
import javax.swing.*;
import javax.swing.text.html.*;
import javax.swing.text.*;
import java.net.URL;
import java.io.*;

/** Diese Klasse ist fr das Zeichnen der Linien zustndig. Hier werden die Methoden,
  * die in Drawable definiert sind implementiert.
  * Zum Zeichnen einer Linie ist folgender Ablauf notwendig:<br>
  * Klick auf den Startpunkt, Aufziehen der Linie, Loslassen der Mousetaste am
  * Endpunkt.
  */
public class jdLineGenerator implements Drawable {

  /** die Koordinaten der Linie als Zwischenspeicher. */
  private double x1, y1, x2, y2;

  /** Boolscher Wert der angibt, ob gengend Informationen zu zeichnen der Linie
   * gesammelt wurden.
   */
  private boolean finished = false;

  /** Die Linie, die whren des Zeichnens angezeigt wird. */
  private Line2D line = new Line2D.Double();

  /** Die GlassPane, auf der der Vorgang des Zeichnens dargestellt wird. */
  private GlassPane glasspane = null;

  /** Die JEditorPane, auf der die Online-Hilfe dargestellt werden kann. */
  private JEditorPane helpPane = null;

  /** Defaultkonstruktor. */
  public jdLineGenerator() {
    try {
      jbInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  /** Diese Methode wird aufgerufen, wenn der Benutzer die Mouse auf der GlassPane
   * drckt.
   */
  public boolean handleMousePressed(MouseEvent e) {
    // Zuweisen des Hilfetextes.
    setHelp("<html>Halten Sie jetzt die Mousetaste gedrckt und ziehen Sie die Linie bis zu ihrem Endpunkt auf.<br>" +
    "Lassen Sie dort die Mousetaste los um das Zeichnen der Linie abzuschlieen.</HTML>");

    // Es sind nicht genug Informationen gesammelt.
    finished = false;

    // Die Koordinaten auf den Punkt setzen, der angeklickt wurde.
    x1 = e.getX();
    y1 = e.getY();
    x2 = x1;
    y2 = y1;

    // Die Linie setzen.
    line.setLine(x1,y1,x2,y2);

    // Wenn es eine Glasspane gibt, die Linie dort anzeigen.
    if (glasspane != null) glasspane.addShape(line);

    // False zurckgeben, da noch nicht genug Informationen gesammelt wurden.
    return false;
  }

  /** Diese Methode wird aufgerufen, wenn die Mouse mit gedrckter Mousetaste ber
   * die GlassPane gezogen wird.
   */
  public boolean handleMouseDragged(MouseEvent e) {
    // Zuweisen des zweiten Punktes der Linie.
    x2 = e.getX();
    y2 = e.getY();

    // zweiten Punkt der Linie zuweisen.
    line.setLine(x1,y1,x2,y2);

    // die GlassPane zum neuzeichnen auffordern.
    glasspane.refresh();

    // False zurckgeben, da noch nicht genug Informationen gesammelt wurden.
    return false;
  }

  /** Diese Methode wird aufgerufen, wenn die Mouse ohne Taste ber die GlassPane
   * bewegt wird.
   */
  public boolean handleMouseMoved(MouseEvent e) {
    return false;
  }

  /** Diese Methode wird aufgerufen, wenn die Mousetaste losgelassen wurde. */
  public boolean handleMouseReleased(MouseEvent e) {
    // Zuweisen des Hilfetextes.
    setHelp("<html>Nun knnen Sie eine weitere Linie zeichnen, klicken Sie dazu " +
    "mit der linken Mousetaste auf den Anfangspunkt der Linie:</html>");

    // Zuweisen des Linienendpunktes.
    x2 = e.getX();
    y2 = e.getY();

    // EndPunkt auf Linie bertragen.
    line.setLine(x1,y1,x2,y2);

    // Wenn es eine GlassPane gibt, dann die Linie von dort wieder entfernen.
    if (glasspane != null) glasspane.remShape(line);

    // True zurckliefern, da nun genug Informationen zum erzeugen einer Linie
    // vorhanden sind, Das finish flag auf true setzen.
    finished = true;
    return true;
  }

  /** Wird aufgerufen, wenn die Mouse geklickt wurde. */
  public boolean handleMouseClicked(MouseEvent e) {
    return false;
  }

  /** Wird aufgerufen, wenn das Zeichnen abgebrochen werden soll. */
  public void handleTermination() {
    // Das finished flag auf false setzen.
    finished = false;
  }

  /** Liefert eine Instanz von jdLine zurck, wenn es aus den gesammelten Informationen
   * mglich war eine Linie zu zeichnen. Ansonsten null.
   * @return Eine Instanz von jdLine, oder null.
   */
  public AbstractGraphObject getInstance() {
    AbstractGraphObject grph = null;

    // Wenn genug Info's dann ein neues jdLine Objekt erzeugen.
    if (finished) grph = new jdLine((Line2D) line.clone());
    return grph;
  }

  /** Wird aufgerufen, um dem Drawable die Mglichkeit zu geben, sich graphisch
   * darzustellen.
   */
  public void setGlassPane(GlassPane glasspane) {
    this.glasspane = glasspane;
  }

  /** Liefert den zum Zeichnen zu verwendenden Cursor zurck. *
   * @return Den zum Zeichnen genutzten Cursor.
   */
  public Cursor supportCursor() {
    return Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR);
  }

  private void jbInit() throws Exception {
  }

  /** Liefert einen allgemeinen Helpfile, der beschreibt, was eine Linie ist.
   * @return Die URL auf den allgemeinen helpfile.
   */
  public URL getHTMLHelp() {
    return jdLineGenerator.class.getResource("lineGeneratorHelp.html");
  }

  /** Gibt dem Drawable die Mglichkeit online Hilfe anzuzeigen.
   * @param e die JEditorPane, auf der ein Hilfetext im HTML format angezeigt werden
   * kann.
   */
  public void setHTMLHelpPane( JEditorPane e ) {
    this.helpPane = e;
  }

  /** Schreibt einen String auf die helpPane.
   * @param s Der Helpstring im HTML-Format.
   */
  private void setHelp(String s) {
    if (helpPane==null) return;
    helpPane.setText(s);
  }
}